/*
 *  
 */
/********************************************************************
 * Copyright (C) 1990-1994, GMD Darmstadt. All rights reserved.     *
 *                                                                  *
 *                                                                  *
 *                         NOTICE                                   *
 *                                                                  *
 *    Acquisition, use, and distribution of this module             *
 *    and related materials are subject to restrictions             *
 *    mentioned in each volume of the documentation.                *
 *                                                                  *
 ********************************************************************/


#ifndef _SEC_CONFIG_
#define _SEC_CONFIG_

#include "config.h"

#include "CONFIG.h"      /* generated with config/configure */


                                                
/*
 * Miscellaneous
 */


/* Define one one of the three USE_ parameters below in order to be able
 * to read PINs longer than 8 characters. If none of the USE_ parameters is
 * defined, PINs are read with getpasswd(), restricting the length of the PIN
 * to 8 characters.
 */

#define USE_TERMIO              /* Use termio for terminal settings               */
/* #define USE_TERMIOS          /* Use termios for terminal settings              */
/* #define USE_SGTTY            /* Use sgtty for terminal settings                */
 
#define NDBM                    /* ndbm database functions are available:         */

#ifndef SIZEOFINT               /* this is the size of int in bits. 
                                   Possible values: 16, 32                        */
#define SIZEOFINT 32
#endif
#ifndef SIZEOFLONG              /* this is the size of long in bits.
                                   Possible values: 32, 64                        */
#define SIZEOFLONG 32
#endif


/*
 * Definitions for secxlock
 */

#define DEF_FILESEARCHPATH "/%T/%N%S"

#ifndef	_CONFIG_                /* This is the ISODE config.h                     */
#define	_CONFIG_

/*
 * The following are the ISODE ICR1 config.h definitions needed for the
 * integrated ISODE routines
 */

/* 
 * $Log: sec_config.h,v $
 * Revision 1.1  1999/10/28 11:31:23  luicat
 * Initial revision
 *
 * Revision 2.0  1998/10/22 08:38:47  anidel
 * Initial revision
 *
 * Revision 2.0  1998/10/22 08:38:29  anidel
 * Initial revision
 *
 * Revision 1.2  1998/01/19 14:49:04  anidel
 * *** empty log message ***
 *
 * Revision 1.1  1997/12/12 00:05:12  anidel
 * Initial revision
 *
 * Revision 1.7  1995/01/25  13:46:38  giehl
 * define ICR1 und/oder define ICR2 wird nun von
 * configure ausgefuehrt; configure schreibt dies
 * defines in CONFIG.h - wird included!
 *
 * Revision 1.6  1995/01/17  10:07:49  giehl
 * Define ICR2 added
 *
 * Revision 1.5  1994/11/10  16:38:21  schneiw
 * WLNG16, WLNG32 und WLNG ersetzt durch SIZEOFINT und SIZEOFLONG.
 *
 * Revision 1.4  1994/11/02  10:44:39  surkau
 * Header replaced by Id
 *
 * Revision 1.3  1994/11/02  09:39:08  surkau
 * SecuDE-4.4.a0
 *
 * Revision 1.2  1994/11/01  08:53:59  surkau
 * ifdef XXX removed
 *
 * Revision 1.1  1994/10/17  13:57:35  surkau
 * Initial revision
 *
 * Revision 10.0  1993/02/18  20:26:44  awy
 * Release IC-R1
 *
 */


#ifdef LINUX
#define BSD42
#undef SYS5
#endif

#undef ROS			/* Ridge's emulation of UNIX */

/***********************************************************************
 *
 * Now some feature that variously implemented.
 *
 */

#define	WRITEV			/* has writev(2) system call*/
#define	GETDENTS		/* has getdirent(2) call */
#undef	SCANDIR			/* has scandir(3) & alphasort(3) */
#define	NFS			/* network filesystem -- has getdirentries */
#define TEMPNAM                 /* has tempnam (3) call */
#define	VSPRINTF		/* has vprintf(3s) routines */
#define	GETCWD			/* has getcwd(3) */

/***********************************************************************
 *
 * What networking options are available
 *
 */

#define	TCP			/* has TCP/IP for TP0 over RFC1006 */
#ifdef TCP
#  define SOCKETS		/*   provided by sockets */
#    undef WIN			/*      emulated via WIN/TCP for SVR2 */
#    undef BIND			/*      if BIND is used (the gethostent struct
				        has an h_addrlist field in <netdb.h>)
				        you may need to add "-lresolv" to the
				        LIBSYS definition in Make.defs */
#  undef EXOS			/*   provided by EXOS 8044 card */
#  undef WIN_TLI		/*   provided by WIN/TCP STREAMS */
#  undef TLI_TCP		/*   provided by TLI */
#  undef SELECT			/*   use select(2) even without SOCKETS */
#endif /* TCP */


#undef X25			/* X.25/CONS support for TP0 */
#ifdef X25
#  undef SUN_X25		/*   provided by SunNet 7.0 */
#  undef SUN_X25_HACK		/*      to enable local subaddress masking
					(see comments in
					lib/ll/converge/x25addr.c) */
#  undef CAMTEC			/*   using CAMTEC interface */
#  undef CAMTEC_CCL		/*   using CAMTEC interface with sockets */
#  undef UBC_X25		/*   using UBC X.25 */
#  undef UBC_X25_WRITEV		/*   able to use writev for X.25,
                                        not all versions of UBC code
                                        support this. */
#  undef ULTRIX_X25		/*   using Ultrix X.25 */
#  undef ULTRIX_X25_DEMSA	/*   X.25 access to DEC DEMSA x25router 2000 */
#  undef HPUX_X25		/*   using HP-UX X.25 */
#  undef CCUR_X25		/*   using CCUR X.25 */
#endif /* X25 */


#undef TP4			/* has TP4 (or other kernal COTS) */
#ifdef TP4
#  undef SUN_TP4		/*   provided by SunLink OSI */
#    undef SUNLINK_5_2		/*     define if using SunLink OSI release 5.2
				       or greater */
#    undef SUNLINK_6_0		/*     define if using SunLink OSI release 6.0
				       or greater */
#    undef SUNLINK_7_0		/*     define if using SunNet OSI release 7.0
				       or greater */
#  undef BSD_TP4		/*   provided by UCB/UWisc */
#  undef TLI_TP			/*   provided by TLI interface */
#    undef ICL_TLI		/*     using the ICL DRS6000 TLI interface */
#    undef RTnet_R02		/*     using the CCUR's RTnet R02 or later */
#  undef XTI_TP			/*     provided by XTI interface */
#endif /* TP4 */

#undef TLI_POLL			/* Use poll(2) instead of select(2) */


/***********************************************************************
 *
 * Directory Services
 * 
 *     By default, support for both the OSI Directory and the stub DSE is
 *     compiled.  However, to enable access to the Directory (via a
 *     lightweight access protocol), the isotailor variable ns_enabled
 *     must be turned "on".  If the nameservice fails, then the stub DSE
 *     will be consulted.
 * 
 *     While not advised at this point you can disable fallback to the stub
 *     DSE by also choosing NOSTUB.  Disabling the fallback mode implies
 *     that all name to address resolution will depend on the DSE and will
 *     fail if it is not operational. 
 * 
 *     You can also disable use of the OSI Directory by NO_USE_DSE.
 *     If you are not building the asn1-libraries then you can remove
 *     a dependency on them, used in encoding AETs, with NO_STUB_AET.
 * 
 *     It does not make much sense to define both NOSTUB & NO_USE_DSE.
 */

#undef	NOSTUB		/* do not use stub DSE when resolving names */
#undef	NO_USE_DSE	/* do not use real OSI Directory */
#undef  NO_STUB_AET     /* disable generation of AETs in
			   stub directory service */

/***********************************************************************
 *
 * Finally some ISODE options
 *
 */

#undef USE_PP			/* Have PP so use it in DUAs */
#undef OSISEC			/* Have OSISEC security package */
#undef OSISEC2			/* Have OSISEC2 security package */
#define SECUDE			/* Have SecuDE security package */
#undef DMS			/* Have DMS security package */

#define NOGOSIP			/* tsapd(8) will only handle calls
				   discriminated at Transport (TSEL);
				   undef'ing this is deprecated */

#endif /* _CONFIG_ */

#endif /* _SEC_CONFIG_ */
