#define AFDBFILE "/home/anidel/secude/etc/.af-db"
#define ALIASDIR "/home/anidel/secude/etc"
#define TESTKEYS_PSE "/home/anidel/secude/etc/.testkeys"
#define ICR1
#define PEPYPATH
#define __MALLOC_0_RETURNS_NULL
#define LINUX
#define BSD42
#define BSD43
