/*
 *	Transparent Cryptographic File System (TCFS) for NetBSD 
 *	Author and mantainer: 	Luigi Catuogno [luicat@tcfs.unisa.it]
 *	
 *	references:		http://tcfs.dia.unisa.it
 *				tcfs-bsd@tcfs.unisa.it
 */

/*
 *	Base utility set v0.1
 *
 *	  $Source: /home/luicat/text/source/include/RCS/tcfsdefines.h,v $
 *	   $State: Exp $
 *	$Revision: 1.1 $
 *	  $Author: luicat $
 *	    $Date: 1999/10/28 11:31:23 $
 *
 */

/* RCS_HEADER_ENDS_HERE */



#ifndef KEYSIZE
#define KEYSIZE 16
#endif
#define UUKEYSIZE        100
#define TRUE             1
#define FALSE            0
#define ONE              1 /* decrement key counter by 1 */
#define ALL              0 /* decrement key counter to 0 */
#define SET              1 /* set permanent flag */
#define UNSET            0 /* unset permanent flag */
#define NONE             2 /* no one of the previous */

#define USERKEY          0
#define GROUPKEY         1

#ifndef _PASSWORD_LEN
#define _PASSWORD_LEN	128
#endif


